/* SPDX-License-Identifier: (LGPL-2.1 OR BSD-2-Clause) */

/* THIS FILE IS AUTOGENERATED! */
#ifndef __RECURSION_SKEL_H__
#define __RECURSION_SKEL_H__

#include <errno.h>
#include <stdlib.h>
#include <bpf/libbpf.h>

struct recursion {
	struct bpf_object_skeleton *skeleton;
	struct bpf_object *obj;
	struct {
		struct bpf_map *hash1;
		struct bpf_map *hash2;
		struct bpf_map *bss;
	} maps;
	struct {
		struct bpf_program *on_lookup;
	} progs;
	struct {
		struct bpf_link *on_lookup;
	} links;
	struct recursion__bss {
		int pass1;
		int pass2;
	} *bss;
};

static void
recursion__destroy(struct recursion *obj)
{
	if (!obj)
		return;
	if (obj->skeleton)
		bpf_object__destroy_skeleton(obj->skeleton);
	free(obj);
}

static inline int
recursion__create_skeleton(struct recursion *obj);

static inline struct recursion *
recursion__open_opts(const struct bpf_object_open_opts *opts)
{
	struct recursion *obj;
	int err;

	obj = (struct recursion *)calloc(1, sizeof(*obj));
	if (!obj) {
		errno = ENOMEM;
		return NULL;
	}

	err = recursion__create_skeleton(obj);
	err = err ?: bpf_object__open_skeleton(obj->skeleton, opts);
	if (err)
		goto err_out;

	return obj;
err_out:
	recursion__destroy(obj);
	errno = -err;
	return NULL;
}

static inline struct recursion *
recursion__open(void)
{
	return recursion__open_opts(NULL);
}

static inline int
recursion__load(struct recursion *obj)
{
	return bpf_object__load_skeleton(obj->skeleton);
}

static inline struct recursion *
recursion__open_and_load(void)
{
	struct recursion *obj;
	int err;

	obj = recursion__open();
	if (!obj)
		return NULL;
	err = recursion__load(obj);
	if (err) {
		recursion__destroy(obj);
		errno = -err;
		return NULL;
	}
	return obj;
}

static inline int
recursion__attach(struct recursion *obj)
{
	return bpf_object__attach_skeleton(obj->skeleton);
}

static inline void
recursion__detach(struct recursion *obj)
{
	return bpf_object__detach_skeleton(obj->skeleton);
}

static inline int
recursion__create_skeleton(struct recursion *obj)
{
	struct bpf_object_skeleton *s;

	s = (struct bpf_object_skeleton *)calloc(1, sizeof(*s));
	if (!s)
		goto err;
	obj->skeleton = s;

	s->sz = sizeof(*s);
	s->name = "recursion";
	s->obj = &obj->obj;

	/* maps */
	s->map_cnt = 3;
	s->map_skel_sz = sizeof(*s->maps);
	s->maps = (struct bpf_map_skeleton *)calloc(s->map_cnt, s->map_skel_sz);
	if (!s->maps)
		goto err;

	s->maps[0].name = "hash1";
	s->maps[0].map = &obj->maps.hash1;

	s->maps[1].name = "hash2";
	s->maps[1].map = &obj->maps.hash2;

	s->maps[2].name = "recursio.bss";
	s->maps[2].map = &obj->maps.bss;
	s->maps[2].mmaped = (void **)&obj->bss;

	/* programs */
	s->prog_cnt = 1;
	s->prog_skel_sz = sizeof(*s->progs);
	s->progs = (struct bpf_prog_skeleton *)calloc(s->prog_cnt, s->prog_skel_sz);
	if (!s->progs)
		goto err;

	s->progs[0].name = "on_lookup";
	s->progs[0].prog = &obj->progs.on_lookup;
	s->progs[0].link = &obj->links.on_lookup;

	s->data_sz = 2696;
	s->data = (void *)"\
\x7f\x45\x4c\x46\x02\x01\x01\0\0\0\0\0\0\0\0\0\x01\0\xf7\0\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x08\x08\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\x40\0\x0a\0\
\x01\0\0\x2e\x73\x74\x72\x74\x61\x62\0\x2e\x73\x79\x6d\x74\x61\x62\0\x66\x65\
\x6e\x74\x72\x79\x2f\x5f\x5f\x68\x74\x61\x62\x5f\x6d\x61\x70\x5f\x6c\x6f\x6f\
\x6b\x75\x70\x5f\x65\x6c\x65\x6d\0\x6c\x69\x63\x65\x6e\x73\x65\0\x2e\x62\x73\
\x73\0\x2e\x6d\x61\x70\x73\0\x72\x65\x63\x75\x72\x73\x69\x6f\x6e\x2e\x63\0\x6f\
\x6e\x5f\x6c\x6f\x6f\x6b\x75\x70\0\x68\x61\x73\x68\x31\0\x70\x61\x73\x73\x31\0\
\x68\x61\x73\x68\x32\0\x70\x61\x73\x73\x32\0\x5f\x6c\x69\x63\x65\x6e\x73\x65\0\
\x2e\x72\x65\x6c\x66\x65\x6e\x74\x72\x79\x2f\x5f\x5f\x68\x74\x61\x62\x5f\x6d\
\x61\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\0\x2e\x42\x54\x46\0\
\x2e\x42\x54\x46\x2e\x65\x78\x74\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x42\0\0\0\x04\0\xf1\xff\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\x03\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x4e\0\0\0\x12\0\x03\0\0\0\0\0\
\0\0\0\0\xd8\0\0\0\0\0\0\0\x58\0\0\0\x11\0\x06\0\0\0\0\0\0\0\0\0\x20\0\0\0\0\0\
\0\0\x5e\0\0\0\x11\0\x05\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\x64\0\0\0\x11\0\
\x06\0\x20\0\0\0\0\0\0\0\x20\0\0\0\0\0\0\0\x6a\0\0\0\x11\0\x05\0\x04\0\0\0\0\0\
\0\0\x04\0\0\0\0\0\0\0\x70\0\0\0\x11\0\x04\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\
\x79\x11\0\0\0\0\0\0\xb7\x02\0\0\0\0\0\0\x63\x2a\xfc\xff\0\0\0\0\x18\x02\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x5d\x21\x06\0\0\0\0\0\x18\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x61\x12\0\0\0\0\0\0\x07\x02\0\0\x01\0\0\0\x63\x21\0\0\0\0\0\0\x05\0\x0d\0\0\
\0\0\0\x18\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x5d\x21\x0a\0\0\0\0\0\x18\x01\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x61\x12\0\0\0\0\0\0\x07\x02\0\0\x01\0\0\0\x63\x21\0\0\0\
\0\0\0\xbf\xa2\0\0\0\0\0\0\x07\x02\0\0\xfc\xff\xff\xff\x18\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x85\0\0\0\x01\0\0\0\xb7\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\x47\x50\
\x4c\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x18\0\0\0\0\0\
\0\0\x01\0\0\0\x04\0\0\0\x30\0\0\0\0\0\0\0\x01\0\0\0\x05\0\0\0\x60\0\0\0\0\0\0\
\0\x01\0\0\0\x06\0\0\0\x78\0\0\0\0\0\0\0\x01\0\0\0\x07\0\0\0\xb0\0\0\0\0\0\0\0\
\x01\0\0\0\x06\0\0\0\x9f\xeb\x01\0\x18\0\0\0\0\0\0\0\xbc\x01\0\0\xbc\x01\0\0\
\x08\x02\0\0\0\0\0\0\0\0\0\x02\x03\0\0\0\x01\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\0\
\x01\0\0\0\0\0\0\0\x03\0\0\0\0\x02\0\0\0\x04\0\0\0\x01\0\0\0\x05\0\0\0\0\0\0\
\x01\x04\0\0\0\x20\0\0\0\0\0\0\0\0\0\0\x02\x02\0\0\0\0\0\0\0\0\0\0\x02\x07\0\0\
\0\x19\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\x01\0\0\0\0\x04\0\0\x04\x20\0\0\0\x1e\
\0\0\0\x01\0\0\0\0\0\0\0\x23\0\0\0\x01\0\0\0\x40\0\0\0\x2f\0\0\0\x05\0\0\0\x80\
\0\0\0\x33\0\0\0\x06\0\0\0\xc0\0\0\0\x39\0\0\0\0\0\0\x0e\x08\0\0\0\x01\0\0\0\
\x3f\0\0\0\0\0\0\x0e\x08\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\x02\x0c\0\0\0\x45\0\0\0\
\0\0\0\x01\x08\0\0\0\x40\0\0\0\0\0\0\0\x01\0\0\x0d\x02\0\0\0\x58\0\0\0\x0b\0\0\
\0\x5c\0\0\0\x01\0\0\x0c\x0d\0\0\0\x66\0\0\0\0\0\0\x01\x01\0\0\0\x08\0\0\x01\0\
\0\0\0\0\0\0\x03\0\0\0\0\x0f\0\0\0\x04\0\0\0\x04\0\0\0\x6b\0\0\0\0\0\0\x0e\x10\
\0\0\0\x01\0\0\0\x74\0\0\0\0\0\0\x0e\x02\0\0\0\x01\0\0\0\x7a\0\0\0\0\0\0\x0e\
\x02\0\0\0\x01\0\0\0\xd7\x01\0\0\x01\0\0\x0f\x04\0\0\0\x11\0\0\0\0\0\0\0\x04\0\
\0\0\xdf\x01\0\0\x02\0\0\x0f\x08\0\0\0\x12\0\0\0\0\0\0\0\x04\0\0\0\x13\0\0\0\
\x04\0\0\0\x04\0\0\0\xe4\x01\0\0\x02\0\0\x0f\x40\0\0\0\x09\0\0\0\0\0\0\0\x20\0\
\0\0\x0a\0\0\0\x20\0\0\0\x20\0\0\0\0\x69\x6e\x74\0\x5f\x5f\x41\x52\x52\x41\x59\
\x5f\x53\x49\x5a\x45\x5f\x54\x59\x50\x45\x5f\x5f\0\x6c\x6f\x6e\x67\0\x74\x79\
\x70\x65\0\x6d\x61\x78\x5f\x65\x6e\x74\x72\x69\x65\x73\0\x6b\x65\x79\0\x76\x61\
\x6c\x75\x65\0\x68\x61\x73\x68\x31\0\x68\x61\x73\x68\x32\0\x75\x6e\x73\x69\x67\
\x6e\x65\x64\x20\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\0\x63\x74\x78\0\x6f\x6e\
\x5f\x6c\x6f\x6f\x6b\x75\x70\0\x63\x68\x61\x72\0\x5f\x6c\x69\x63\x65\x6e\x73\
\x65\0\x70\x61\x73\x73\x31\0\x70\x61\x73\x73\x32\0\x2f\x62\x75\x69\x6c\x64\x64\
\x69\x72\x2f\x62\x75\x69\x6c\x64\x2f\x42\x55\x49\x4c\x44\x2f\x6b\x65\x72\x6e\
\x65\x6c\x2d\x72\x74\x2d\x34\x2e\x31\x38\x2e\x30\x2d\x35\x35\x33\x2e\x35\x38\
\x2e\x31\x2e\x72\x74\x37\x2e\x33\x39\x39\x2e\x65\x6c\x38\x5f\x31\x30\x2f\x6c\
\x69\x6e\x75\x78\x2d\x34\x2e\x31\x38\x2e\x30\x2d\x35\x35\x33\x2e\x35\x38\x2e\
\x31\x2e\x72\x74\x37\x2e\x33\x39\x39\x2e\x65\x6c\x38\x5f\x31\x30\x2e\x78\x38\
\x36\x5f\x36\x34\x2f\x74\x6f\x6f\x6c\x73\x2f\x74\x65\x73\x74\x69\x6e\x67\x2f\
\x73\x65\x6c\x66\x74\x65\x73\x74\x73\x2f\x62\x70\x66\x2f\x70\x72\x6f\x67\x73\
\x2f\x72\x65\x63\x75\x72\x73\x69\x6f\x6e\x2e\x63\0\x69\x6e\x74\x20\x42\x50\x46\
\x5f\x50\x52\x4f\x47\x28\x6f\x6e\x5f\x6c\x6f\x6f\x6b\x75\x70\x2c\x20\x73\x74\
\x72\x75\x63\x74\x20\x62\x70\x66\x5f\x6d\x61\x70\x20\x2a\x6d\x61\x70\x29\0\x09\
\x69\x6e\x74\x20\x6b\x65\x79\x20\x3d\x20\x30\x3b\0\x09\x69\x66\x20\x28\x6d\x61\
\x70\x20\x3d\x3d\x20\x28\x76\x6f\x69\x64\x20\x2a\x29\x26\x68\x61\x73\x68\x31\
\x29\x20\x7b\0\x09\x09\x70\x61\x73\x73\x31\x2b\x2b\x3b\0\x09\x09\x72\x65\x74\
\x75\x72\x6e\x20\x30\x3b\0\x09\x69\x66\x20\x28\x6d\x61\x70\x20\x3d\x3d\x20\x28\
\x76\x6f\x69\x64\x20\x2a\x29\x26\x68\x61\x73\x68\x32\x29\x20\x7b\0\x09\x09\x70\
\x61\x73\x73\x32\x2b\x2b\x3b\0\x09\x09\x62\x70\x66\x5f\x6d\x61\x70\x5f\x6c\x6f\
\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\x28\x26\x68\x61\x73\x68\x32\x2c\x20\x26\
\x6b\x65\x79\x29\x3b\0\x6c\x69\x63\x65\x6e\x73\x65\0\x2e\x62\x73\x73\0\x2e\x6d\
\x61\x70\x73\0\x66\x65\x6e\x74\x72\x79\x2f\x5f\x5f\x68\x74\x61\x62\x5f\x6d\x61\
\x70\x5f\x6c\x6f\x6f\x6b\x75\x70\x5f\x65\x6c\x65\x6d\0\0\0\0\0\x9f\xeb\x01\0\
\x20\0\0\0\0\0\0\0\x14\0\0\0\x14\0\0\0\xbc\0\0\0\xd0\0\0\0\0\0\0\0\x08\0\0\0\
\xea\x01\0\0\x01\0\0\0\0\0\0\0\x0e\0\0\0\x10\0\0\0\xea\x01\0\0\x0b\0\0\0\0\0\0\
\0\x80\0\0\0\x19\x01\0\0\x05\x70\0\0\x10\0\0\0\x80\0\0\0\x46\x01\0\0\x06\x78\0\
\0\x18\0\0\0\x80\0\0\0\x54\x01\0\0\x0a\x80\0\0\x28\0\0\0\x80\0\0\0\x54\x01\0\0\
\x06\x80\0\0\x30\0\0\0\x80\0\0\0\x72\x01\0\0\x08\x84\0\0\x58\0\0\0\x80\0\0\0\
\x7d\x01\0\0\x03\x88\0\0\x60\0\0\0\x80\0\0\0\x89\x01\0\0\x0a\x90\0\0\x70\0\0\0\
\x80\0\0\0\x89\x01\0\0\x06\x90\0\0\x78\0\0\0\x80\0\0\0\xa7\x01\0\0\x08\x94\0\0\
\xb0\0\0\0\x80\0\0\0\xb2\x01\0\0\x03\xa4\0\0\xc8\0\0\0\x80\0\0\0\x19\x01\0\0\
\x05\x70\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\x03\0\
\0\0\x20\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\0\0\xa9\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x09\0\0\0\x02\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\xf0\0\0\0\0\0\0\0\xd8\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\x08\
\0\0\0\0\0\0\0\x18\0\0\0\0\0\0\0\x11\0\0\0\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\xc8\x01\0\0\0\0\0\0\xd8\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\x2f\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa0\
\x02\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\x37\0\0\0\x08\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa4\x02\0\0\0\0\0\
\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x04\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x3c\0\0\
\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa8\x02\0\0\0\0\0\0\x40\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x79\0\0\0\x09\0\0\0\
\x40\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xe8\x02\0\0\0\0\0\0\x50\0\0\0\0\0\0\0\x02\0\
\0\0\x03\0\0\0\x08\0\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x9b\0\0\0\x01\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x38\x03\0\0\0\0\0\0\xdc\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xa0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x18\x07\0\0\0\0\0\0\xf0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0";

	return 0;
err:
	bpf_object__destroy_skeleton(s);
	return -ENOMEM;
}

#endif /* __RECURSION_SKEL_H__ */
